local ADDON_NAME, SHARED = ...;

local LibStub = LibStub;
local A = SHARED[1];

local LDB = LibStub:GetLibrary("LibDataBroker-1.1");
local AceDB = LibStub("AceDB-3.0")

local ICON_PATTERN_14 = "|T%s:14:14:0:0|t";
local BUFFY_ICON = "Interface\\Icons\\spell_arcane_invocation";

local TEX_BUFFY_ICON = ICON_PATTERN_14:format(BUFFY_ICON);

function A:InitializeDatabase()
	local defaults = {
		global = {
			Enabled = true,
			ShowInCombat = false,
			ShowWhileMounted = false,
			ShowBuffStatus = true,
			
			Class = {
				Rogue = {
					EnableLethal = true,
					EnableNonlethal = true,
				},
				Shaman = {
					RemindShield = true,	
				},
				Hunter = {
					EnableLoneWolf = true,	
				},
			},
		},
	};
	
	self.db = AceDB:New(ADDON_NAME .. "DB", defaults);
end

local classOptions = {
	["ROGUE"]	= {
		text = "Rogue Options",
		hasArrow = true,
		notCheckable = true,
		menuList = {
			{
				text = "Rogue Options", isTitle = true, notCheckable = true,
			},
			{
				text = "Enable Alert for Lethal Poisons",
				func = function() A.db.global.Class.Rogue.EnableLethal = not A.db.global.Class.Rogue.EnableLethal; A:UpdateBuffs(); end,
				checked = function() return A.db.global.Class.Rogue.EnableLethal; end,
				isNotRadio = true,
			},
			{
				text = "Enable Alert for Non-Lethal Poisons",
				func = function() A.db.global.Class.Rogue.EnableNonlethal = not A.db.global.Class.Rogue.EnableNonlethal; A:UpdateBuffs(); end,
				checked = function() return A.db.global.Class.Rogue.EnableNonlethal; end,
				isNotRadio = true,
			},
		},
	},
	["SHAMAN"]	= {
		text = "Shaman Options",
		hasArrow = true,
		notCheckable = true,
		menuList = {
			{
				text = "Shaman Options", isTitle = true, notCheckable = true,
			},
			{
				text = "Enable Alert for Shaman Shield Buff",
				func = function() A.db.global.Class.Shaman.RemindShield = not A.db.global.Class.Shaman.RemindShield; A:UpdateBuffs(); end,
				checked = function() return A.db.global.Class.Shaman.RemindShield; end,
				isNotRadio = true,
			},
		},
	},
	["HUNTER"]	= {
		text = "Hunter Options",
		hasArrow = true,
		notCheckable = true,
		menuList = {
			{
				text = "Hunter Options", isTitle = true, notCheckable = true,
			},
			{
				text = "Enable Alerts for Lone Wolf Buffs",
				func = function() A.db.global.Class.Hunter.EnableLoneWolf = not A.db.global.Class.Hunter.EnableLoneWolf; A:UpdateBuffs(); end,
				checked = function() return A.db.global.Class.Hunter.EnableLoneWolf; end,
				isNotRadio = true,
			},
		},
	},
};

function A:GetDatabrokerMenuData()
	local legibleClass, class = UnitClass("player");
	
	local data = {
		{
			text = "Buffy Options", isTitle = true, notCheckable = true,
		},
		{
			text = "Buffy Notifications Enabled",
			func = function() self.db.global.Enabled = not self.db.global.Enabled; A:UpdateBuffs(); end,
			checked = function() return self.db.global.Enabled; end,
			isNotRadio = true,
		},
		{
			text = "Show Notifications in Combat",
			func = function() self.db.global.ShowInCombat = not self.db.global.ShowInCombat; A:UpdateBuffs(); end,
			checked = function() return self.db.global.ShowInCombat; end,
			isNotRadio = true,
		},
		{
			text = "Show Notifications While Flying or in a Vehicle",
			func = function() self.db.global.ShowWhileMounted = not self.db.global.ShowWhileMounted; A:UpdateBuffs(); end,
			checked = function() return self.db.global.ShowWhileMounted; end,
			isNotRadio = true,
		},
		{
			text = "Show Raid Buff Status on Databroker",
			func = function() self.db.global.ShowBuffStatus = not self.db.global.ShowBuffStatus; A:UpdateBuffs(); end,
			checked = function() return self.db.global.ShowBuffStatus; end,
			isNotRadio = true,
		},
	};
	
	if(classOptions[class] ~= nil) then
		tinsert(data, classOptions[class]);
	else
		tinsert(data, {
			text = "No options for " .. legibleClass,
			notCheckable = true,
			disabled = true,
		});
	end
	
	tinsert(data, 
	{
		text = "Close Menu",
		func = function() CloseMenus() end,
		notCheckable = true,
	});
	
	return data;
end

function A:OpenContextMenu(parentframe)
	if(not A.ContextMenu) then
		A.ContextMenu = CreateFrame("Frame", "PetBuddyContextMenuFrame", parentframe, "UIDropDownMenuTemplate");
	end
	
	A.ContextMenu:SetPoint("BOTTOM", parentframe, "CENTER", 0, 5);
	EasyMenu(A:GetDatabrokerMenuData(), A.ContextMenu, "cursor", 0, 0, "MENU", 5);
end

function A:UpdateDatabrokerText()
	if(self.db.global.ShowBuffStatus) then
		local buffs = A:ScanCurrentBuffs();
		local foundBuffs = 0;
		local numBuffTypes = #A.BUFF_TYPES;
		
		for category, data in pairs(buffs) do
			if(data ~= false) then
				foundBuffs = foundBuffs + 1;
			end
		end
		
		A.databroker.text = string.format("%d/%d |cffffcc00buffs|r", foundBuffs, numBuffTypes);
	else
		A.databroker.text = "Buffy";
	end
end

local function GetClassColor(class)
	return (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[class];
end

-- function A:UnitIsPet(unit)
-- 	return UnitPlayerControlled(unit) and not UnitIsPlayer(unit) and
-- 		  (UnitPlayerOrPetInParty(unit) or UnitPlayerOrPetInRaid(unit) or UnitIsUnit(unit, "pet"));
-- end



function A:InitializeDatabroker()
	A.databroker = LDB:NewDataObject(ADDON_NAME, {
		type = "data source",
		label = "Buffy",
		text = "Buffy",
		icon = BUFFY_ICON,
		OnClick = function(frame, button)
			if(button == "RightButton") then
				GameTooltip:Hide();
				A:OpenContextMenu(frame);
			end
		end,
		OnTooltipShow = function(tooltip)
			if not tooltip or not tooltip.AddLine then return end
			tooltip:AddLine(TEX_BUFFY_ICON .. " |cffffffffBuffy|r");
			tooltip:AddLine(" ");
			
			if(not self.db.global.Enabled) then
				tooltip:AddLine("|cffe05754Notifications Currently Disabled|r");
				tooltip:AddLine(" ");
			end
			
			tooltip:AddLine("|cff78dbfcCurrent Buffs|r");
			
			local buffs = A:ScanCurrentBuffs();
			
			for index, category in ipairs(A.BUFF_TYPES) do
				local data = buffs[category];
				local categoryName = A.BUFF_TYPE_NAMES[category];
				
				if(data == false) then
					tooltip:AddDoubleLine(categoryName, "|cffffffffMissing|r");
				else
					local casterName = data.casterName;
					local classColor = "ffffffff";
					
					local isPet, masterName, petName = A:UnitIsPet(casterName);
					if(not isPet) then
						classColor = GetClassColor(data.casterClass).colorStr;
					else
						casterName = string.format("%s|r's pet", masterName);
						
						local _, class = UnitClass(masterName);
						classColor = GetClassColor(class).colorStr;
					end
					
					local buffname, _, bufficon = GetSpellInfo(data.buff);
					
					tooltip:AddDoubleLine(
						categoryName,
						string.format("%s %s (|c%s%s|r)", ICON_PATTERN_14:format(bufficon), buffname, classColor, casterName),
						nil, nil, nil, 1, 1, 1
					);
				end
			end
			
			tooltip:AddLine(" ");
			tooltip:AddLine("Right-Click |cffffffffShow options|h");
			
			-- Addon.tooltip_open = true;
		end,
		OnLeave = function(frame)
			-- Addon.tooltip_open = false;
		end,
	});

	A:UpdateDatabrokerText();
end